import Option, { Some } from '../option.ts';
import {
	AbstractFileType,
	defaultHighlightOptions,
	FileLang,
	HighlightingOptions,
} from './base.ts';

export class RustFile extends AbstractFileType {
	public readonly name: FileLang = FileLang.Rust;
	public readonly singleLineComment = Some('//');
	public readonly multiLineCommentStart: Option<string> = Some('/*');
	public readonly multiLineCommentEnd: Option<string> = Some('*/');
	public readonly keywords1 = [
		'continue',
		'return',
		'static',
		'struct',
		'unsafe',
		'break',
		'const',
		'crate',
		'extern',
		'match',
		'super',
		'trait',
		'where',
		'else',
		'enum',
		'false',
		'impl',
		'loop',
		'move',
		'self',
		'type',
		'while',
		'for',
		'let',
		'mod',
		'pub',
		'ref',
		'true',
		'use',
		'mut',
		'as',
		'fn',
		'if',
		'in',
	];
	public readonly keywords2 = [
		'DoubleEndedIterator',
		'ExactSizeIterator',
		'IntoIterator',
		'PartialOrd',
		'PartialEq',
		'Iterator',
		'ToString',
		'Default',
		'ToOwned',
		'Extend',
		'FnOnce',
		'Option',
		'String',
		'AsMut',
		'AsRef',
		'Clone',
		'Debug',
		'FnMut',
		'Sized',
		'Unpin',
		'array',
		'isize',
		'usize',
		'&str',
		'Copy',
		'Drop',
		'From',
		'Into',
		'None',
		'Self',
		'Send',
		'Some',
		'Sync',
		'bool',
		'char',
		'i128',
		'u128',
		'Box',
		'Err',
		'Ord',
		'Vec',
		'dyn',
		'f32',
		'f64',
		'i16',
		'i32',
		'i64',
		'str',
		'u16',
		'u32',
		'u64',
		'Eq',
		'Fn',
		'Ok',
		'i8',
		'u8',
		'&mut self',
		'&mut',
		'&self',
		'self',
	];
	public readonly operators = [
		'||=',
		'>>=',
		'<=>',
		'<<=',
		'&&=',
		'**=',
		'..=',
		'...',
		'||',
		'|=',
		'>>',
		'>=',
		'=>',
		'==',
		'<=',
		'<<',
		'<-',
		'+=',
		'++',
		'^=',
		'%=',
		'&=',
		'&&',
		'/=',
		'*=',
		'**',
		'..',
		'!=',
		':=',
		'::',
		'->',
		'-=',
		'--',
		'~',
		'|',
		'>',
		'=',
		'<',
		'+',
		'^',
		'%',
		'&',
		'*',
		'.',
		'!',
		':',
		';',
		',',
		'-',
	];
	public readonly hlOptions: HighlightingOptions = {
		...defaultHighlightOptions,
		characters: true,
		binNumbers: true,
		hexNumbers: true,
	};
}
